==> drd-8.x-3.4.tar.gz: drd/src/DomainLocalCopy.php <==
<?php

namespace Drupal\drd;

use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Database\Database;
use Drupal\drd\Entity\DomainInterface;
use Symfony\Component\Filesystem\Filesystem;

/**
 * Build a local copy of a domain into a given working directory.
 */
class DomainLocalCopy {

  /** @var ImmutableConfig $drdConfig */
  protected $drdConfig;

  /** @var string[] */
  protected $log = [];

  /** @var string */
  protected $drupalDir;

  /** @var string */
  protected $workingDir;

  /** @var string */
  protected $settingsDir;

  /** @var DomainInterface */
  protected $domain;

  /** @var array */
  protected $domainGlobals;

  /** @var array */
  protected $domainSettings;

  /** @var int */
  protected $coreVersion;

  /**
   * @return string[]
   */
  public function getLog(): array {
    return $this->log;
  }

  /**
   * @param string $drupalDir
   *
   * @return DomainLocalCopy
   */
  public function setDrupalDirectory($drupalDir) {
    $this->drupalDir = $drupalDir;
    return $this;
  }

  /**
   * @param string $workingDir
   *
   * @return DomainLocalCopy
   */
  public function setWorkingDirectory($workingDir) {
    $this->workingDir = $workingDir;
    return $this;
  }

  /**
   * @param DomainInterface $domain
   *
   * @return DomainLocalCopy
   */
  public function setDomain($domain) {
    $this->drdConfig = \Drupal::config('drd.general');
    $this->domain = $domain;
    $this->domainGlobals = $this->domain->getRemoteGlobals();
    $this->domainSettings = $this->domain->getRemoteSettings();
    $this->coreVersion = $this->domain->getCore()->getDrupalRelease()->getMajor()->getCoreVersion();
    $this->settingsDir = implode(DIRECTORY_SEPARATOR, [
      $this->drupalDir,
      'sites',
      $this->domain->getLocalUrl(),
    ]);

    // Create or update sites.php
    $sites_file = implode(DIRECTORY_SEPARATOR, [
      $this->drupalDir,
      'sites',
      'sites.php',
    ]);
    if (!file_exists($sites_file)) {
      file_put_contents($sites_file, '<?php');
    }
    file_put_contents(
      $sites_file,
      PHP_EOL . '$sites["' . $this->domain->getLocalUrl() . '"] = "' . $this->domain->getLocalUrl() . '";',
      FILE_APPEND
    );

    return $this;
  }

  /**
   * @return bool
   */
  public function setup() {
    $databases = $this->domain->database();
    if (empty($databases)) {
      return FALSE;
    }
    foreach ($databases as $key => $targets) {
      foreach ($targets as $target => $def) {
        $databases[$key][$target]['database'] = $this->buildDatabase($key, $target, $def);
      }
    }

    $options = [
      'drd' => [
        'db' => [
          'user' => $this->drdConfig->get('local.db.user'),
          'pass' => $this->drdConfig->get('local.db.pass'),
        ],
      ],
      'databases' => $databases,
      'database_config' => Database::getConnectionInfo()['default'],
      'globals' => $this->domainGlobals,
      'settings' => $this->domainSettings,
      'tempdir' => drupal_realpath(file_directory_temp()),
      'url' => $this->domain->getLocalUrl(),
    ];

    $fs = new Filesystem();
    $fs->mkdir($this->settingsDir);

    $templatefilename = drupal_get_path('module', 'drd') . '/templates/DomainLocalCopy.v' . $this->coreVersion . '.settings.php.twig';

    $twig_loader = new \Twig_Loader_Array([]);
    $twig = new \Twig_Environment($twig_loader);
    $twig_loader->setTemplate('settings', file_get_contents($templatefilename));
    $rendered = $twig->render('settings', $options);
    file_put_contents($this->settingsDir . DIRECTORY_SEPARATOR . 'settings.php', $rendered);

    $fs->mkdir($this->drupalDir . DIRECTORY_SEPARATOR . $this->domainSettings['file_public_path']);
    $fs->mkdir($this->drupalDir . DIRECTORY_SEPARATOR . $this->domainSettings['file_private_path']);
    foreach ($this->domainGlobals['config_directories'] as $config_directory) {
      $fs->mkdir($this->drupalDir . DIRECTORY_SEPARATOR . $config_directory);
    }
    return TRUE;
  }

  /**
   * @param string $key
   * @param string $target
   * @param array $def
   *
   * @return string
   */
  private function buildDatabase($key, $target, $def) {
    $database = implode('_', [
      'drd',
      'dump',
      $this->domain->id(),
      $key,
      $target,
    ]);
    $output = [];

    $config = Database::getConnectionInfo()['default'];
    $credentialsfile = drupal_realpath(\Drupal::service('file_system')->tempnam('temporary://', 'mysql'));

    $cmd = [
      'mysql',
      '--defaults-extra-file=' . $credentialsfile,
    ];
    $credentials = [
      '[mysql]',
      'host = ' . $config['host'],
      'port = ' . $config['port'],
      'user = ' . $this->drdConfig->get('local.db.user'),
      'password = "' . $this->drdConfig->get('local.db.pass'),
    ];

    file_put_contents($credentialsfile, implode("\n", $credentials));
    chmod($credentialsfile, 0600);

    $prepare = array_merge($cmd, [
      '--execute="DROP DATABASE IF EXISTS ' . $database . '; CREATE DATABASE ' . $database . ';"'
    ]);
    exec(implode(' ', $prepare), $output, $ret);
    if ($ret !== 0) {
      $output[] = 'Can not prepare database.';
    }
    else {
      $import = array_merge($cmd, [
        $database,
        '<' . $def['file'],
      ]);
      exec(implode(' ', $import), $output, $ret);
      if ($ret !== 0) {
        $output[] = 'Can not import database.';
      }
    }

    unlink($credentialsfile);

    foreach ($output as $item) {
      $this->log[] = $item;
    }

    return $database;
  }

}

==> drd-8.x-3.5.tar.gz: drd/src/DomainLocalCopy.php <==
<?php

namespace Drupal\drd;

use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Database\Database;
use Drupal\drd\Entity\DomainInterface;
use Symfony\Component\Filesystem\Filesystem;

/**
 * Build a local copy of a domain into a given working directory.
 */
class DomainLocalCopy {

  /** @var ImmutableConfig $drdConfig */
  protected $drdConfig;

  /** @var string[] */
  protected $log = [];

  /** @var string */
  protected $drupalDir;

  /** @var string */
  protected $workingDir;

  /** @var string */
  protected $settingsDir;

  /** @var DomainInterface */
  protected $domain;

  /** @var array */
  protected $domainGlobals;

  /** @var array */
  protected $domainSettings;

  /** @var int */
  protected $coreVersion;

  /**
   * @return string[]
   */
  public function getLog() {
    return $this->log;
  }

  /**
   * @param string $drupalDir
   *
   * @return DomainLocalCopy
   */
  public function setDrupalDirectory($drupalDir) {
    $this->drupalDir = $drupalDir;
    return $this;
  }

  /**
   * @param string $workingDir
   *
   * @return DomainLocalCopy
   */
  public function setWorkingDirectory($workingDir) {
    $this->workingDir = $workingDir;
    return $this;
  }

  /**
   * @param DomainInterface $domain
   *
   * @return DomainLocalCopy
   */
  public function setDomain($domain) {
    $this->drdConfig = \Drupal::config('drd.general');
    $this->domain = $domain;
    $this->domainGlobals = $this->domain->getRemoteGlobals();
    $this->domainSettings = $this->domain->getRemoteSettings();
    $this->coreVersion = $this->domain->getCore()->getDrupalRelease()->getMajor()->getCoreVersion();
    $this->settingsDir = implode(DIRECTORY_SEPARATOR, [
      $this->drupalDir,
      'sites',
      $this->domain->getLocalUrl(),
    ]);

    // Create or update sites.php
    $sites_file = implode(DIRECTORY_SEPARATOR, [
      $this->drupalDir,
      'sites',
      'sites.php',
    ]);
    if (!file_exists($sites_file)) {
      file_put_contents($sites_file, '<?php');
    }
    file_put_contents(
      $sites_file,
      PHP_EOL . '$sites["' . $this->domain->getLocalUrl() . '"] = "' . $this->domain->getLocalUrl() . '";',
      FILE_APPEND
    );

    return $this;
  }

  /**
   * @return bool
   */
  public function setup() {
    $databases = $this->domain->database();
    if (empty($databases)) {
      return FALSE;
    }
    foreach ($databases as $key => $targets) {
      foreach ($targets as $target => $def) {
        $databases[$key][$target]['database'] = $this->buildDatabase($key, $target, $def);
      }
    }

    $options = [
      'drd' => [
        'db' => [
          'user' => $this->drdConfig->get('local.db.user'),
          'pass' => $this->drdConfig->get('local.db.pass'),
        ],
      ],
      'databases' => $databases,
      'database_config' => Database::getConnectionInfo()['default'],
      'globals' => $this->domainGlobals,
      'settings' => $this->domainSettings,
      'tempdir' => drupal_realpath(file_directory_temp()),
      'url' => $this->domain->getLocalUrl(),
    ];

    $fs = new Filesystem();
    $fs->mkdir($this->settingsDir);

    $templatefilename = drupal_get_path('module', 'drd') . '/templates/DomainLocalCopy.v' . $this->coreVersion . '.settings.php.twig';

    $twig_loader = new \Twig_Loader_Array([]);
    $twig = new \Twig_Environment($twig_loader);
    $twig_loader->setTemplate('settings', file_get_contents($templatefilename));
    $rendered = $twig->render('settings', $options);
    file_put_contents($this->settingsDir . DIRECTORY_SEPARATOR . 'settings.php', $rendered);

    $fs->mkdir($this->drupalDir . DIRECTORY_SEPARATOR . $this->domainSettings['file_public_path']);
    $fs->mkdir($this->drupalDir . DIRECTORY_SEPARATOR . $this->domainSettings['file_private_path']);
    foreach ($this->domainGlobals['config_directories'] as $config_directory) {
      $fs->mkdir($this->drupalDir . DIRECTORY_SEPARATOR . $config_directory);
    }
    return TRUE;
  }

  /**
   * @param string $key
   * @param string $target
   * @param array $def
   *
   * @return string
   */
  private function buildDatabase($key, $target, $def) {
    $database = implode('_', [
      'drd',
      'dump',
      $this->domain->id(),
      $key,
      $target,
    ]);
    $output = [];

    $config = Database::getConnectionInfo()['default'];
    $credentialsfile = drupal_realpath(\Drupal::service('file_system')->tempnam('temporary://', 'mysql'));

    $cmd = [
      'mysql',
      '--defaults-extra-file=' . $credentialsfile,
    ];
    $credentials = [
      '[mysql]',
      'host = ' . $config['host'],
      'port = ' . $config['port'],
      'user = ' . $this->drdConfig->get('local.db.user'),
      'password = "' . $this->drdConfig->get('local.db.pass'),
    ];

    file_put_contents($credentialsfile, implode("\n", $credentials));
    chmod($credentialsfile, 0600);

    $prepare = array_merge($cmd, [
      '--execute="DROP DATABASE IF EXISTS ' . $database . '; CREATE DATABASE ' . $database . ';"'
    ]);
    exec(implode(' ', $prepare), $output, $ret);
    if ($ret !== 0) {
      $output[] = 'Can not prepare database.';
    }
    else {
      $import = array_merge($cmd, [
        $database,
        '<' . $def['file'],
      ]);
      exec(implode(' ', $import), $output, $ret);
      if ($ret !== 0) {
        $output[] = 'Can not import database.';
      }
    }

    unlink($credentialsfile);

    foreach ($output as $item) {
      $this->log[] = $item;
    }

    return $database;
  }

}
